﻿using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace plexdata.Controls
{
    public class SystemMenu
    {
        private const uint MAX_POSITION = 0x0FFF;
        private IntPtr hSysMenu = IntPtr.Zero;

        private SystemMenu()
            : base()
        {
        }

        public static SystemMenu FromWindow(Form window)
        {
            SystemMenu result = new SystemMenu();

            result.hSysMenu = GetSystemMenu(window.Handle, false);

            if (result.hSysMenu == IntPtr.Zero)
            {
                throw new NullReferenceException(String.Format("GetSystemMenu() failed with error code {0} (0x{0:X8})", GetLastError()));
            }
            else
            {
                return result;
            }
        }

        public IntPtr Handle
        {
            get { return this.hSysMenu; }
        }

        public bool IsValid
        {
            get { return (this.Handle != IntPtr.Zero); }
        }

        public bool AddSeparator()
        {
            if (this.IsValid)
            {
                return AppendMenu(this.Handle, MF_BYPOSITION | MF_SEPARATOR, UIntPtr.Zero, null);
            }
            else
            {
                return false;
            }
        }

        public bool DelItem(uint item)
        {
            if (this.IsValid)
            {
                uint flags = this.IsPosition(item) ? MF_BYPOSITION : MF_BYCOMMAND;
                return DeleteMenu(this.Handle, item, flags);
            }
            else
            {
                return false;
            }
        }

        public bool AddItem(uint item, string label)
        {
            if (this.IsValid && this.IsPosition(item))
            {
                return AppendMenu(this.Handle, MF_BYPOSITION | MF_STRING, new UIntPtr(item), label);
            }
            else
            {
                return false;
            }
        }

        private bool IsPosition(uint item)
        {
            return (item <= SystemMenu.MAX_POSITION);
        }

        #region System Menu Command Values (winuser.h)

        public static uint WM_SYSCOMMAND = 0x0112;

        public static uint SC_SIZE = 0xF000;
        public static uint SC_MOVE = 0xF010;
        public static uint SC_MINIMIZE = 0xF020;
        public static uint SC_MAXIMIZE = 0xF030;
        public static uint SC_NEXTWINDOW = 0xF040;
        public static uint SC_PREVWINDOW = 0xF050;
        public static uint SC_CLOSE = 0xF060;
        public static uint SC_VSCROLL = 0xF070;
        public static uint SC_HSCROLL = 0xF080;
        public static uint SC_MOUSEMENU = 0xF090;
        public static uint SC_KEYMENU = 0xF100;
        public static uint SC_ARRANGE = 0xF110;
        public static uint SC_RESTORE = 0xF120;
        public static uint SC_TASKLIST = 0xF130;
        public static uint SC_SCREENSAVE = 0xF140;
        public static uint SC_HOTKEY = 0xF150;
        public static uint SC_DEFAULT = 0xF160;
        public static uint SC_MONITORPOWER = 0xF170;
        public static uint SC_CONTEXTHELP = 0xF180;
        public static uint SC_SEPARATOR = 0xF00F;

        #endregion

        #region Flags to add/check/enable menu items (winuser.h)

        public static uint MF_INSERT = 0x00000000;
        public static uint MF_CHANGE = 0x00000080;
        public static uint MF_APPEND = 0x00000100;
        public static uint MF_DELETE = 0x00000200;
        public static uint MF_REMOVE = 0x00001000;
        public static uint MF_BYCOMMAND = 0x00000000;
        public static uint MF_BYPOSITION = 0x00000400;
        public static uint MF_SEPARATOR = 0x00000800;
        public static uint MF_ENABLED = 0x00000000;
        public static uint MF_GRAYED = 0x00000001;
        public static uint MF_DISABLED = 0x00000002;
        public static uint MF_UNCHECKED = 0x00000000;
        public static uint MF_CHECKED = 0x00000008;
        public static uint MF_USECHECKBITMAPS = 0x00000200;
        public static uint MF_STRING = 0x00000000;
        public static uint MF_BITMAP = 0x00000004;
        public static uint MF_OWNERDRAW = 0x00000100;
        public static uint MF_POPUP = 0x00000010;
        public static uint MF_MENUBARBREAK = 0x00000020;
        public static uint MF_MENUBREAK = 0x00000040;
        public static uint MF_UNHILITE = 0x00000000;
        public static uint MF_HILITE = 0x00000080;
        public static uint MF_DEFAULT = 0x00001000;
        public static uint MF_SYSMENU = 0x00002000;
        public static uint MF_HELP = 0x00004000;
        public static uint MF_RIGHTJUSTIFY = 0x00004000;
        public static uint MF_MOUSESELECT = 0x00008000;
        public static uint MF_END = 0x00000080;

        #endregion

        #region Win32 API access functions.

        [DllImport("user32", SetLastError = true, CallingConvention = CallingConvention.Winapi)]
        private static extern IntPtr GetSystemMenu(IntPtr hWnd, bool revert);

        [DllImport("user32", SetLastError = true, CallingConvention = CallingConvention.Winapi)]
        private static extern bool DeleteMenu(IntPtr hMenu, uint position, uint flags);

        [DllImport("user32", SetLastError = true, CallingConvention = CallingConvention.Winapi)]
        private static extern bool AppendMenu(IntPtr hMenu, uint flags, UIntPtr idNewItem, string lpNewItem);

        [DllImport("user32", SetLastError = true, CallingConvention = CallingConvention.Winapi)]
        private static extern bool EnableMenuItem(IntPtr hMenu, uint item, uint flags);

        [DllImport("kernel32", CallingConvention = CallingConvention.Winapi)]
        private static extern uint GetLastError();

        #endregion
    }
}
